# coding: utf-8
# 2021/3/15 @ tongshiwei
import re
from collections import OrderedDict
import torch


def save_params(filename, net: torch.nn.Module, select=None):
    params = net.state_dict()

    if select is None:
        if select is not None:
            pattern = re.compile(select)
            params = OrderedDict({name: value for name, value in params.items() if
                                  pattern.match(name)})

    torch.save(params, filename)



# coding: utf-8
# 2021/3/15 @ tongshiwei

def get_total_update_steps(update_epoch, batches_per_epoch, epoch_update_freq=1):
    return int(update_epoch * batches_per_epoch * epoch_update_freq)


def get_factor_lr_params(base_lr, update_epoch, batches_per_epoch, epoch_update_freq=1, stop_lr=None, discount=None):
    assert epoch_update_freq

    total_update_steps = get_total_update_steps(
        update_epoch,
        batches_per_epoch,
        epoch_update_freq
    )
    step = batches_per_epoch // epoch_update_freq
    stop_lr, discount = _stop_lr(base_lr, stop_lr, discount)
    return step, pow(discount, step / total_update_steps), stop_lr


def _stop_lr(base_lr, stop_lr=None, discount=None):
    assert stop_lr or discount
    if stop_lr:
        return stop_lr, discount
    elif discount:
        return base_lr * discount, discount
